==> openerp-7.0.2404.1.tar.gz: openerp-7.0.2404.1/openerp/service/cron.py <==
# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2011 OpenERP SA (<http://www.openerp.com>)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

""" Cron jobs scheduling

Cron jobs are defined in the ir_cron table/model. This module deals with all
cron jobs, for all databases of a single OpenERP server instance.

"""

import logging
import threading
import time
from datetime import datetime

import openerp
import openerp.tools.config as config

from openerp.cli.server import preload_registry

_logger = logging.getLogger(__name__)

SLEEP_INTERVAL = 60 # 1 min

def _db_list():
    if config['db_name']:
        db_names = config['db_name'].split(',')
    else:
        db_names = openerp.netsvc.ExportService._services['db'].exp_list(True)
    return db_names

def preload_registries():
    """preload registry for all databases who have enabled cron jobs"""

    for db_name in _db_list():
        status = True
        log = _logger.info
        if not openerp.modules.registry.RegistryManager.registries.get(db_name):
            cr = None
            try:
                cr = openerp.sql_db.db_connect(db_name).cursor()
                cr.execute("SELECT EXISTS( \
                    SELECT 1 FROM information_schema.tables \
                        WHERE table_catalog=%s AND \
                        table_schema='public' AND \
                        table_name=%s)", (db_name, 'ir_cron'))
                if cr.fetchone()[0]:
                    cr.execute("SELECT EXISTS( \
                        SELECT 1 FROM ir_cron \
                            WHERE active = True) AS preload")
                    res = cr.fetchone()
                    if res and res != None:
                        if not preload_registry(db_name):
                            status = False
                            log = _logger.error
                    log("cron: preloaded registry for database %s: [succeeded=%s]" % (db_name, status))
            except Exception, e:
                _logger.error("Exception when preloading registry for database %s: %s" % (db_name, e))
            finally:
                if not cr.closed:
                    cr.close()

def cron_runner(cron_id):
    while True:
        acquired = False
        time.sleep(SLEEP_INTERVAL + cron_id) # Steve Reich timing style
        if config['list_db'] == 'internal':
            preload_registries()
        registries = openerp.modules.registry.RegistryManager.registries
        _logger.debug('cron%d polling for jobs', cron_id)
        for db_name, registry in registries.items():
            while True and registry.ready:
                # acquired = openerp.addons.base.ir.ir_cron.ir_cron._acquire_job(db_name)
                # TODO why isnt openerp.addons.base defined ?
                import sys
                base = sys.modules['addons.base']
                try:
                    acquired = base.ir.ir_cron.ir_cron._acquire_job(db_name)
                except OperationalError, e:
                    _logger.exception('cron%d cannot acquire job for database %s: %s' % (cron_id, db_name, e))
                finally:
                    if not acquired:
                        openerp.modules.registry.RegistryManager.delete(db_name)
                        break
                # dont keep cursors in multi database mode
                if len(db_names) > 1:
                    openerp.sql_db.close_db(db_name)

def start_service():
    """ Start the above runner function in a daemon thread.

    The thread is a typical daemon thread: it will never quit and must be
    terminated when the main process exits - with no consequence (the processing
    threads it spawns are not marked daemon).

    """
    
    # Force call to strptime just before starting the cron thread
    # to prevent time.strptime AttributeError within the thread.
    # See: http://bugs.python.org/issue7980
    datetime.strptime('2012-01-01', '%Y-%m-%d')

    for i in range(openerp.tools.config['max_cron_threads']):
        def target():
            cron_runner(i)
        t = threading.Thread(target=target, name="openerp.service.cron.cron%d" % i)
        t.setDaemon(True)
        t.start()
        _logger.debug("cron%d started!" % i)

def stop_service():
    pass

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

==> openerp-7.0-20130219-002614.tar.gz: openerp-7.0-20130219-002614/openerp/service/cron.py <==
#!/usr/bin/env python
# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2011 OpenERP SA (<http://www.openerp.com>)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

""" Cron jobs scheduling

Cron jobs are defined in the ir_cron table/model. This module deals with all
cron jobs, for all databases of a single OpenERP server instance.

"""

import logging
import threading
import time

import openerp

_logger = logging.getLogger(__name__)

SLEEP_INTERVAL = 60 # 1 min

def cron_runner(number):
    while True:
        time.sleep(SLEEP_INTERVAL + number) # Steve Reich timing style
        registries = openerp.modules.registry.RegistryManager.registries
        _logger.debug('cron%d polling for jobs', number)
        for db_name, registry in registries.items():
            while True and registry.ready:
                # acquired = openerp.addons.base.ir.ir_cron.ir_cron._acquire_job(db_name)
                # TODO why isnt openerp.addons.base defined ?
                import sys
                base = sys.modules['addons.base']
                acquired = base.ir.ir_cron.ir_cron._acquire_job(db_name)
                if not acquired:
                    break

def start_service():
    """ Start the above runner function in a daemon thread.

    The thread is a typical daemon thread: it will never quit and must be
    terminated when the main process exits - with no consequence (the processing
    threads it spawns are not marked daemon).

    """
    for i in range(openerp.tools.config['max_cron_threads']):
        def target():
            cron_runner(i)
        t = threading.Thread(target=target, name="openerp.service.cron.cron%d" % i)
        t.setDaemon(True)
        t.start()
        _logger.debug("cron%d started!" % i)

def stop_service():
    pass

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
